#!/usr/bin/perl -w

use Kpathsea;

use Getopt::Long;

use strict;

my $dpi = 600;
my $braces_to_expand = '';
my $path_to_expand = '';
my $var_to_expand = '';
my $path_to_show = '';

my $user_format_string = '';
my $user_path = '';
my $progname = '';
my $version = 0;
my $interactive = 0;
my $help = 0;
my $mktexs = {};
my $mode = '';
my $must_exist = 0;
my $show_all = 0;

my $debug = 0;

my $kpse = new Kpathsea;

my $user_format = $kpse->{last_format};

sub lookup {
  my ($name) = @_;
  my ($ret,$local_dpi,$glyph_ret);
  if ($user_path) {
    if ($show_all) {
      $ret = $kpse->all_path_search ($user_path, $name, $must_exist);
    } else {
      $ret = $kpse->path_search ($user_path, $name, $must_exist);
    }
  } else {
    my $fmt = $kpse->find_format ($name, $user_format);
    if($fmt == $kpse->{last_format}) {
      $fmt = $kpse->{tex_format};
    }
    if ($fmt == $kpse->{pk_format} ||
        $fmt == $kpse->{gf_format} ||
        $fmt == $kpse->{any_glyph_format}) {
      $local_dpi = $kpse->find_dpi ($name);
      if (!$local_dpi) {
	$local_dpi = $dpi;
      }
      ($ret,$glyph_ret) = $kpse->find_glyph ($kpse->remove_suffix ($name),$local_dpi, $fmt);
    } else {
      $ret = $kpse->find_file ($name, $fmt, $must_exist, $show_all);
    }
  }
  if (defined $ret) {
    if ($show_all) {
      print join("\n",@$ret) . "\n";
    } else {
      print $ret . "\n";
    }
  }
  return !(defined $ret);
}


sub maketex_option {
  my ($a,$b) = @_;
  if ($a eq 'mktex') {
	$kpse->maketex_option($b,1);
  } elsif ($a eq 'no-mktex') {
	$kpse->maketex_option($b,0);
  }
}


my $ret = GetOptions(
		     'debug=i' => \$debug,
		     'D=i' => \$dpi,
		     'dpi=i' => \$dpi,
		     'expand-braces=s' => \$braces_to_expand,
		     'expand-path=s' => \$path_to_expand,
		     'expand-var=s' => \$var_to_expand,
		     'format=s' => \$user_format_string,
		     'help' => \$help,
		     'interactive' => \$interactive,
		     'mktex=s' => \&maketex_option,
		     'no-mktex=s' => \&maketex_option,
		     'mode=s' => \$mode,
		     'must-exist' => \$must_exist,
		     'path=s' => \$user_path,
		     'progname=s' => \$progname,
		     'showpath=s' => \$path_to_show,
		     'version' => \$version,
		     'all' => \$show_all
		    );

if ($debug) {
  $kpse->{kpathsea_debug} |= $debug;
}

if ($help)  {
  print "Usage: $0 [OPTION]... [FILENAME]...\n";
  print "  Standalone path lookup and expansion for Kpathsea.

-debug=NUM             set debugging flags.
-D, -dpi=NUM           use a base resolution of NUM; default 600.
-expand-braces=STRING  output variable and brace expansion of STRING.
-expand-path=STRING    output complete path expansion of STRING.
-expand-var=STRING     output variable expansion of STRING.
-format=NAME           use file type NAME (see list below).
-help                  print this message and exit.
-interactive           ask for additional filenames to look up.
[-no]-mktex=FMT        disable/enable mktexFMT generation (FMT=pk/mf/tex/tfm).
-mode=STRING           set device name for \$MAKETEX_MODE to STRING;
                       no default.
-must-exist            search the disk as well as ls-R if necessary
-path=STRING           search in the path STRING.
-progname=STRING       set program name to STRING.
-show-path=NAME        output search path for file type NAME (see list below).

-version               print version number and exit.\n";
  print $kpse->{'bug_address'}. "\n";
  $kpse->set_program_name($0,$progname);
  print "Recognized format names and their suffixes:\n";
  $kpse->init_formats();
  my $f=0;
  while ($f < $kpse->{last_format}) {
    print $kpse->format_info_string($f) . "\n";
    $f++;
  }
  exit 0;
}

if ($version) {
   print $kpse->{kpathsea_version_string} . "\n";
   print "Copyright (C) 1997 - 2002 K. Berry & O. Weber.
There is NO warranty.  You may redistribute this software
under the terms of the GNU General Public License.
For more information about these matters, see the files named COPYING.\n";
  exit 0;
}

if ($path_to_show) { 
  $user_format_string = $path_to_show; 
}

if($user_path && $user_format_string) {
  print STDERR "--path ($user_path) and --format ($user_format_string) are mutually exclusive.\n";
  print STDERR "Try `kpsewhich --help' for more information.\n";
  exit 1;
}

unless (@ARGV || $var_to_expand || $braces_to_expand || $path_to_expand || $path_to_show) {
  print STDERR "Missing argument. Try `kpsewhich --help' for more information.\n";
  exit 1;
}

my $fallback_font = '';

$kpse->init_program($0,$progname,$dpi,$mode,$fallback_font);

if ($user_format_string) {
  $user_format = $kpse->find_format($user_format_string);
}

if ($var_to_expand) {
  print $kpse->var_expand($var_to_expand) . "\n";
}

if ($braces_to_expand) {
  print $kpse->brace_expand($braces_to_expand) . "\n";
}

if ($path_to_expand) {
  my $path = $kpse->path_expand($path_to_expand);
  print $path . "\n";
}

if ($path_to_show) {
  if ($user_format != $kpse->{last_format}) {
     $kpse->init_format($user_format);
     print $kpse->{format_info}->[$user_format]->{path} . "\n";
  } else {
    print "kpsewhich: Ignoring unknown file type `$path_to_show'\n";
  }
}

my $unfound = 0;

while (my $optind = shift @ARGV) {
   $unfound += lookup($optind);
}

if ($interactive) {
  while (1) {
    my $name = <STDIN>;
    chomp $name;
    last if !($name) || $name eq 'q' || $name eq 'quit';
    $unfound += lookup($name);
  }
}

exit ($unfound > 255 ? 1 : $unfound);



